using System;
using System.Runtime.Serialization;
using Stardraw;
using Stardraw.Project;
using Stardraw.Control;
using Stardraw.Tcp;

/// <summary>
/// MyClass implements the serial driver for this product
/// It inherits from a SerialInPortInstance so that the driver
/// can simply call Read and Write and need not be concerned
/// about routing through the network, baudrates, parity, etc. etc.
/// It is marked with a [Serializable] attribute so that the driver
/// is saved correctly
/// </summary>
[Serializable]
public class MyClass : TcpInPortInstance
{

	/// <summary>
	/// This is the constructor of MyClass and is called whenever the driver is first created
	/// It takes 2 parameters and must call the base (SerialInPortInstance) with those parameters
	/// so that it may hook up the Product and Ports correctly.
	/// You may do additional initialization here
	/// </summary>
	public MyClass(ProductInstance productInstance, Port port)
		: base(productInstance, port)
	{
	}

	/// <summary>
	/// This method sends requests for all external controls configured in the HAL sample configuration
	/// <summary>
	[Controllable]
	public void RequestAllControlValues()
	{
		RequestSourceSelection();
		RequestSourceSelectionMaximum();
		RequestToneSelection();
		RequestToneSelectionMaximum();
		RequestMuteAllToggle();
		RequestWallOpenToggle();
		RequestLoungeVolumeLevel();
	}

	//
	// Source Selector
	//
	// This is a collection of properties, variables and method the make the Source Selector control
	// available to the Star Draw control application
	//
	private int _sourceSelection;
	private int _sourceSelectionMaximum;
	private String[] _sourceSelectionNames;

	// This is a Number assigned to this control in the HAL configuration
	private readonly uint _sourceSelectionControlNumber = 1;

	// This event fires when the selection number changes for Source Selector
	[Controllable]
	public event EventHandler OnSourceSelectionChanged;

	// This event fires when the number of selections changes for Source Selector
	[Controllable]
	public event EventHandler OnSourceSelectionMaximumChanged;

	// This event fires when a selection name changes for Source Selector
	[Controllable]
	public event EventHandler OnSourceSelectionNamesChanged;

	/// <summary>
	/// This property sets or gets the current selection for Source Selector
	/// The range is 0 to SourceSelectionMaximum
	/// </summary>
	[Controllable]
	public int SourceSelection
	{
		get { return _sourceSelection; }
		set
		{
			// If this value is changing
			if (_sourceSelection != value)
			{
				// update the local state
				updateSourceSelection(value);

				// update the remote state
				setSelection(_sourceSelectionControlNumber, value);
			}
		}
	}

	/// <summary>
	/// This method sends a request for the current selection for Source Selector
	/// When HAL sends a response, it is processed by updateSourceSelection()
	/// </summary>
	[Controllable]
	public void RequestSourceSelection()
	{
		Logger.Debug("Requesting SourceSelection");
		requestSelection(_sourceSelectionControlNumber);
	}

	/// <summary>
	/// This property gets the current maximum selection for Source Selector
	/// It is not settable by the StarDraw application
	/// </summary>
	/// <returns>The maximum selection value for Source Selector</returns>
	[Controllable]
	public int SourceSelectionMaximum
	{
		get { return _sourceSelectionMaximum; }
	}

	/// <summary>
	/// This method sends a request for the current selection maximum for Source Selector
	/// When HAL sends a response, it is processed by updateSourceSelectionMaximum()
	/// </summary>
	[Controllable]
	public void RequestSourceSelectionMaximum()
	{
		Logger.Debug("Requesting SourceSelection maximum");
		requestSelectionLink(_sourceSelectionControlNumber);
	}

	/// <summary>
	/// This property gets the current array of selection names for Source Selector 
	/// These is not settable by the StarDraw application
	/// </summary>
	/// <returns>A string array of the selection names for Source Selector</returns>
	[Controllable]
	public String[] SourceSelectionNames
	{
		get { return _sourceSelectionNames; }
	}

	/// <summary>
	/// This method sends a request for all of the selection names for Source Selector
	/// When HAL sends the responses (one for each name), each is processed by updateSourceSelectionName()
	/// </summary>
	[Controllable]
	public void RequestSourceSelectionNames()
	{
		Logger.Debug("Requesting SourceSelection names");
		for (int selection = 0; selection <= _sourceSelectionMaximum; selection++)
		{
			requestSelectionName(_sourceSelectionControlNumber, selection);
		}
	}

	/// <summary>
	/// This method updates the value of SourceSelection and fires the OnSourceSelectionChanged event
	/// </summary>
	/// <param name="value">The new selection value</param>
	private void updateSourceSelection(int value)
	{

		_sourceSelection = value;
		Logger.Debug("Updating SourceSelection to " + _sourceSelection);

		if (OnSourceSelectionChanged != null)
		{
			OnSourceSelectionChanged(this, EventArgs.Empty);
		}
	}

	/// <summary>
	/// This method updates the value of SourceSelectionMaximum and fires the OnSourceSelectionMaximumChanged event
	/// </summary>
	/// <param name="numberOfSelections">The number of selections currently defined for Source Selector</param>
	private void updateSourceSelectionMaximum(int numberOfSelections)
	{
		if (numberOfSelections > 0)
		{
			// HAL sends number of selections, convert to a maximum index
			_sourceSelectionMaximum = numberOfSelections - 1;
			_sourceSelectionNames = new String[numberOfSelections];
		}
		else
		{
			// This happens when the selector control link is inactive
			_sourceSelectionMaximum = 0;
			_sourceSelectionNames = new String[1];
		}

		Logger.Debug("Updating SourceSelectionMaximum to " + _sourceSelectionMaximum);

		if (OnSourceSelectionMaximumChanged != null)
		{
			OnSourceSelectionMaximumChanged(this, EventArgs.Empty);
		}

		// Request the set of selection names
		RequestSourceSelectionNames();
	}

	/// <summary>
	/// This method updates the value of one of the selection names for SourceSelector and fires the OnSourceSelectionNamesChanged event
	/// </summary>
	/// <param name="selectionNumber">The index of the selection name to set</param>
	/// <param name="name">The new name</param>
	private void updateSourceSelectionName(int selectionNumber, string name)
	{
		if (selectionNumber < _sourceSelectionNames.Length)
		{
			_sourceSelectionNames[selectionNumber] = name;
			Logger.Debug("Updating SourceSelection name " + selectionNumber + " to " + name);

			if (OnSourceSelectionNamesChanged != null)
			{
				OnSourceSelectionNamesChanged(this, EventArgs.Empty);
			}
		}
	}

	//
	// Tone Selector
	//
	// This is a collection of properties, variables and method the make the Tone Selector control
	// available to the Star Draw control application
	//
	private int _toneSelection;
	private int _toneSelectionMaximum;
	private String[] _toneSelectionSelectionNames;

	// This is a Number assigned to this control in the HAL configuration
	private readonly uint _toneSelectionControlNumber = 2;

	// This event fires when the selection number changes for Tone Selector
	[Controllable]
	public event EventHandler OnToneSelectionChanged;

	// This event fires when the number of selections changes for Tone Selector
	[Controllable]
	public event EventHandler OnToneSelectionMaximumChanged;

	// This event fires when a selection name changes for Tone Selector
	[Controllable]
	public event EventHandler OnToneSelectionSelectionNamesChanged;

	/// <summary>
	/// This property sets or gets the current selection for Tone Selector 
	/// The range is 0 to ToneSelectionMaximum
	/// </summary>
	[Controllable]
	public int ToneSelection
	{
		get { return _toneSelection; }
		set
		{
			// If this value is changing
			if (_toneSelection != value)
			{
				// update the local state
				updateToneSelection(value);

				// update the remote state
				setSelection(_toneSelectionControlNumber, value);
			}
		}
	}

	/// <summary>
	/// This method sends a request for the current selection for Tone Selector
	/// When HAL sends a response, it is processed by updateToneSelection()
	/// </summary>
	[Controllable]
	public void RequestToneSelection()
	{
		Logger.Debug("Requesting ToneSelection");
		requestSelection(_toneSelectionControlNumber);
	}

	/// <summary>
	/// This property gets the current maximum selection for Tone Selector
	/// It is not settable by the StarDraw application
	/// </summary>
	/// <returns>The maximum selection value for Tone Selector</returns>
	[Controllable]
	public int ToneSelectionMaximum
	{
		get { return _toneSelectionMaximum; }
	}

	/// <summary>
	/// This method sends a request for the current selection maximum for Tone Selector
	/// When HAL sends a response, it is processed by updateToneSelectionMaximum()
	/// <summary>
	[Controllable]
	public void RequestToneSelectionMaximum()
	{

		Logger.Debug("Requesting ToneSelection maximum");
		requestSelectionLink(_toneSelectionControlNumber);
	}

	/// <summary>
	/// This property gets the current array of selection names for Tone Selector 
	/// These is not settable by the StarDraw application
	/// </summary>
	/// <returns>A string array of the selection names for Tone Selector</returns>
	[Controllable]
	public String[] ToneSelectionNames
	{
		get { return _toneSelectionSelectionNames; }
	}

	/// <summary>
	/// This method sends a request for all of the selection names for Tone Selector
	/// When HAL sends the responses (one for each name), each is processed by updateToneSelectionName()
	/// </summary>
	[Controllable]
	public void RequestToneSelectionNames()
	{
		Logger.Debug("Requesting ToneSelection names");

		for (int selection = 0; selection <= _toneSelectionMaximum; selection++)
		{
			requestSelectionName(_toneSelectionControlNumber, selection);
		}
	}

	/// <summary>
	/// This method updates the value of ToneSelection and fires the OnToneSelectionChanged event
	/// </summary>
	/// <param name="value">The new selection value</param>
	private void updateToneSelection(int value)
	{
		_toneSelection = value;
		Logger.Debug("Updating ToneSelection to " + _sourceSelection);

		if (OnToneSelectionChanged != null)
		{
			OnToneSelectionChanged(this, EventArgs.Empty);
		}
	}

	/// <summary>
	/// This method updates the value of ToneSelectionMaximum and fires the OnToneSelectionMaximumChanged event
	/// </summary>
	/// <param name="numberOfSelections">The number of selections currently defined for Tone Selector</param>
	private void updateToneSelectionMaximum(int numberOfSelections)
	{
		if (numberOfSelections > 0)
		{
			// HAL sends number of selections, convert to a maximum index
			_toneSelectionMaximum = numberOfSelections - 1;
			_toneSelectionSelectionNames = new String[numberOfSelections];
		}
		else
		{
			// This happens when the selector control link is inactive
			_toneSelectionMaximum = 0;
			_toneSelectionSelectionNames = new String[1];
		}

		Logger.Debug("Updating ToneSelectionMaximum to " + _toneSelectionMaximum);

		if (OnToneSelectionMaximumChanged != null)
		{
			OnToneSelectionMaximumChanged(this, EventArgs.Empty);
		}

		// Request the set of selection names
		RequestToneSelectionNames();
	}

	/// <summary>
	/// This method updates the value of one of the selection names for ToneSelector and fires the OnToneSelectionNamesChanged event
	/// </summary>
	/// <param name="selectionNumber">The index of the selection name to set</param>
	/// <param name="name">The new name</param>
	private void updateToneSelectionName(int selectionNumber, string name)
	{
		if (selectionNumber < _toneSelectionSelectionNames.Length)
		{
			_toneSelectionSelectionNames[selectionNumber] = name;
			Logger.Debug("Updating ToneSelection name " + selectionNumber + " to " + name);

			if (OnToneSelectionSelectionNamesChanged != null)
			{
				OnToneSelectionSelectionNamesChanged(this, EventArgs.Empty);
			}
		}
	}

	//
	// Default Levels command
	//
	// This is a collection of properties, variables and method the make the Default Level command control
	// available to the Star Draw control application
	//

	// This is a Number assigned to this control in the HAL configuration
	private readonly uint _defaultLevelsCommandControlNumber = 3;

	/// <summary>
	/// This method sends a command to execute the DefaultLevels command
	/// </summary>
	[Controllable]
	public void ExecDefaultLevelsCommand()
	{
		executeCommand(_defaultLevelsCommandControlNumber);
	}

	//
	// Mute All Toggle
	//
	// This is a collection of properties, variables and method the make the Mute All Toggle control
	// available to the Star Draw control application
	//
	private int _muteAllToggle;

	// This is a Number assigned to this control in the HAL configuration
	private readonly uint _muteAllToggleControlNumber = 4;

	// This event fires when the state of the MuteAllToggle changes
	[Controllable]
	public event EventHandler OnMuteAllToggleChanged;

	/// <summary>
	/// This property sets or gets the current state of the Mute All Toggle control 
	/// The value is either 0 or 1
	/// </summary>
	[Controllable]
	public int MuteAllToggle
	{
		get { return _muteAllToggle; }
		set
		{
			// If this value is changing
			if (_muteAllToggle != value)
			{
				// update the local state
				updateMuteAllToggle(value);

				// update the remote state
				setToggle(_muteAllToggleControlNumber, value);
			}
		}
	}

	/// <summary>
	/// This method sends a request for the current state of the for Mute All Toggle
	/// When HAL sends a response, it is processed by updateMuteAllToggle()
	/// </summary>
	[Controllable]
	public void RequestMuteAllToggle()
	{
		Logger.Debug("Requesting MuteAllToggle");
		requestToggle(_muteAllToggleControlNumber);
	}

	/// <summary>
	/// This method updates the value of Mute All Toggle and fires the OnMuteAllToggleChanged event
	/// </summary>
	/// <param name="value">The new toggle value</param>
	private void updateMuteAllToggle(int value)
	{
		_muteAllToggle = value;
		Logger.Debug("Updating MuteAllToggle to " + _muteAllToggle);

		if (OnMuteAllToggleChanged != null)
		{
			OnMuteAllToggleChanged(this, EventArgs.Empty);
		}
	}

	//
	// Open Wall Toggle
	//
	// This is a collection of properties, variables and method the make the Open Wall Toggle control
	// available to the Star Draw control application
	//
	private int _wallOpenToggle;

	// This is a Number assigned to this control in the HAL configuration
	private readonly uint _wallOpenToggleControlNumber = 5;

	// This event fires when the state of the WallOpenToggle changes
	[Controllable]
	public event EventHandler OnWallOpenToggleChanged;

	/// <summary>
	/// This property sets or gets the current state of the Wall Open Toggle control
	/// The value is either 0 or 1
	/// </summary>
	[Controllable]
	public int WallOpenToggle
	{
		get { return _wallOpenToggle; }
		set
		{
			// If this value is changing
			if (_wallOpenToggle != value)
			{
				// update the local state
				updateWallOpenToggle(value);

				// update the remote state
				setToggle(_wallOpenToggleControlNumber, value);
			}
		}
	}

	/// <summary>
	/// This method sends a request for the current state of the for Wall Open Toggle
	/// When HAL sends a response, it is processed by updateWallOpenToggle()
	/// </summary>
	[Controllable]
	public void RequestWallOpenToggle()
	{
		Logger.Debug("Requesting MuteAllToggle");
		requestToggle(_wallOpenToggleControlNumber);
	}

	/// <summary>
	/// This method updates the value of Wall Open Toggle and fires the OnWallOpenToggleChanged event
	/// </summary>
	/// <param name="value">The new toggle value</param>
	private void updateWallOpenToggle(int value)
	{
		_wallOpenToggle = value;
		Logger.Debug("Updating WallOpenToggle to " + _wallOpenToggle);

		if (OnWallOpenToggleChanged != null)
		{
			OnWallOpenToggleChanged(this, EventArgs.Empty);
		}
	}

	//
	// Lounge Volume Level
	//
	// This is a collection of properties, variables and method the make the Lounge Volume Level control
	// available to the Star Draw control application
	//
	private int _loungeVolumeLevel;

	// This is a Number assigned to this control in the HAL configuration
	private readonly uint _loungeVolumeLevelControlNumber = 6;

	// This event fires when the value of  LoungeVolumeLevel changes
	[Controllable]
	public event EventHandler OnLoungeVolumeLevelChanged;

	/// <summary>
	/// This property sets or gets the current value of the Lounge Volume Level control
	/// The range from 0 to 1000
	/// </summary>
	[Controllable]
	public int LoungeVolumeLevel
	{
		get { return _loungeVolumeLevel; }
		set
		{
			// If this value is changing
			if (_loungeVolumeLevel != value)
			{
				// update the local state
				updateLoungeVolumeLevel(value);

				// update the remote state
				setLevel(_loungeVolumeLevelControlNumber, value);
			}
		}
	}

	/// <summary>
	/// This method sends a request for the current value of the for Lounge Volume Level
	/// When HAL sends a response, it is processed by updateLoungeVolumeLevel()
	/// </summary>
	[Controllable]
	public void RequestLoungeVolumeLevel()
	{
		Logger.Debug("Requesting LoungeVolume");
		requestLevel(_loungeVolumeLevelControlNumber);
	}

	/// <summary>
	/// This method updates the value of Lounge Volume Level and fires the OnLoungeVolumeLevelChanged event
	/// </summary>
	/// <param name="value">The new level value</param>
	private void updateLoungeVolumeLevel(int value)
	{
		_loungeVolumeLevel = value;
		Logger.Debug("Updating LoungeVolumeLevel to " + _loungeVolumeLevel);

		if (OnLoungeVolumeLevelChanged != null)
		{
			OnLoungeVolumeLevelChanged(this, EventArgs.Empty);
		}
	}

	//
	// Control type functions
	//
	// This collection of methods send messages to the HAL external control system. There is a method for each type
	// of message
	//

	/// <summary>
	/// Send a set level message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	/// <param name="value">The level value to set</param> 
	private void setLevel(uint controlNumber, int value)
	{
		string cmd = string.Format("<L&{0}&{1}>", controlNumber, value);
		WriteLine(cmd);

		Logger.Debug("Level - set command sent: " + cmd);
	}

	/// <summary>
	/// Send a request level message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	private void requestLevel(uint controlNumber)
	{
		string cmd = string.Format("<L&{0}>", controlNumber);
		WriteLine(cmd);

		Logger.Debug("Level - request value command sent: " + cmd);
	}

	/// <summary>
	/// Send a set toggle message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	/// <param name="value">The toggle value to set</param> 
	private void setToggle(uint controlNumber, int value)
	{
		string cmd = string.Format("<T&{0}&{1}>", controlNumber, value);
		WriteLine(cmd);

		Logger.Debug("Toggle - set command sent: " + cmd);
	}

	/// <summary>
	/// Send a request toggle message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	private void requestToggle(uint controlNumber)
	{
		string cmd = string.Format("<T&{0}>", controlNumber);
		WriteLine(cmd);

		Logger.Debug("Toggle - request value command sent: " + cmd);
	}

	/// <summary>
	/// Send a command message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	private void executeCommand(uint controlNumber)
	{
		string cmd = string.Format("<C&{0}>", controlNumber);
		WriteLine(cmd);

		Logger.Debug("Command - command sent: " + cmd);
	}

	/// <summary>
	/// Send a set selection message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	/// <param name="value">The selection value to set</param> 
	private void setSelection(uint controlNumber, int value)
	{
		string cmd = string.Format("<S&{0}&{1}>", controlNumber, value);
		WriteLine(cmd);

		Logger.Debug("Selection - set command sent: " + cmd);
	}

	/// <summary>
	/// Send a request selection message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	private void requestSelection(uint controlNumber)
	{
		string cmd = string.Format("<S&{0}>", controlNumber);
		WriteLine(cmd);

		Logger.Debug("Selection - request value command sent: " + cmd);
	}

	/// <summary>
	/// Send a request selection link message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	private void requestSelectionLink(uint controlNumber)
	{
		string cmd = string.Format("<SL&{0}>", controlNumber);
		WriteLine(cmd);

		Logger.Debug("SelectionLink - request value command sent: " + cmd);
	}

	/// <summary>
	/// Send a request selection name message
	/// </summary>
	/// <param name="controlNumber">The HAL control system number for the control</param> 
	/// <param name="selectionNumber">The index of the requested selection name</param> 
	private void requestSelectionName(uint controlNumber, int selectionNumber)
	{
		string cmd = string.Format("<SN&{0}&{1}>", controlNumber, selectionNumber);
		WriteLine(cmd);

		Logger.Debug("SelectionNames - request value command sent: " + cmd);
	}



	//
	// Utility functions
	//

	/// <summary>
	/// Define an enum for the HAL control types
	/// </summary>
	enum MSG_TYPE { None, Level, Select, SelectionName, SelectionLink, Toggle, Command };

	/// <summary>
	/// Define an enum for the action
	/// </summary>
	enum MSG_ACTION { None, NewValue, NewSelectionLink, NewSelectionName };

	/// <summary>
	/// This method parses a message from HAL and processes it. Processing consists of updating the values of any controls.
	/// </summary>
	/// <param name="messageToProcess">The message to process</param>
	private void processMessage(string messageToProcess)
	{
		MSG_TYPE mType = MSG_TYPE.None;
		MSG_ACTION action = MSG_ACTION.None;

		// Split the message into elements, which of which is separated by the '&' character
		char[] delimiter = new char[1] { '&' };
		string[] msgElements = messageToProcess.Split(delimiter);

		string elements = "";
		for (int element = 0; element < msgElements.Length; element++)
		{
			elements += msgElements[element] + " ";
		}
		Logger.Debug("processing message - {0} elements in message: {1}", msgElements.Length, elements);

		// Actionable only if more than 2 elements in the message
		if (msgElements.Length > 2)
		{
			// determine the type amd action based on the first message element
			switch (msgElements[0])
			{
				case "L":
					mType = MSG_TYPE.Level;
					action = MSG_ACTION.NewValue;
					break;

				case "S":
					mType = MSG_TYPE.Select;
					action = MSG_ACTION.NewValue;
					break;

				case "SL":
					mType = MSG_TYPE.SelectionLink;
					action = MSG_ACTION.NewSelectionLink;
					break;

				case "SN":
					mType = MSG_TYPE.SelectionName;
					action = MSG_ACTION.NewSelectionName;
					break;

				case "T":
					mType = MSG_TYPE.Toggle;
					action = MSG_ACTION.NewValue;
					break;
			}

			// Now do the action using the other elements of the message
			switch (action)
			{
				case MSG_ACTION.NewValue:
					{
						int controlID = int.Parse(msgElements[1]);
						int controlValue = int.Parse(msgElements[2]);
						setValue(mType, controlID, controlValue);
					}
					break;

				case MSG_ACTION.NewSelectionLink:
					{
						int controlID = int.Parse(msgElements[1]);
						int nSelections = int.Parse(msgElements[2]);
						setSelectionLink(controlID, nSelections);
					}
					break;

				case MSG_ACTION.NewSelectionName:
					{
						if (msgElements.Length == 4)
						{
							int controlID = int.Parse(msgElements[1]);
							int selectionNumber = int.Parse(msgElements[2]);
							string selectionName = msgElements[3];
							setSelectionName(controlID, selectionNumber, selectionName);
						}
					}
					break;
			}
		}
	}

	/// <summary>
	/// Update a control value according to the type, ID and value. 
	/// </summary>
	/// <param name="mType">The type of message (same as type of control)</param>
	/// <param name="controlNumber">The control number to update</param>
	/// <param name="controlValue">The new value for the control</param>
	private void setValue(MSG_TYPE mType, int controlNumber, int controlValue)
	{
		switch (mType)
		{
			case MSG_TYPE.Select:
				if (controlNumber == _sourceSelectionControlNumber)
				{
					updateSourceSelection(controlValue);
				}
				else if (controlNumber == _toneSelectionControlNumber)
				{
					updateToneSelection(controlValue);
				}
				break;

			case MSG_TYPE.Toggle:
				if (controlNumber == _muteAllToggleControlNumber)
				{
					updateMuteAllToggle(controlValue);
				}
				else if (controlNumber == _wallOpenToggleControlNumber)
				{
					updateWallOpenToggle(controlValue);
				}
				break;

			case MSG_TYPE.Level:
				if (controlNumber == _loungeVolumeLevelControlNumber)
				{
					updateLoungeVolumeLevel(controlValue);
				}
				break;
		}
	}

	/// <summary>
	/// Update a selector control link according to contrl number and value. 
	/// This sets the number of possible selections in a selector control.
	/// </summary>
	/// <param name="controlNumber">The control number to update</param>
	/// <param name="controlValue">The new number of selections for the control</param>
	private void setSelectionLink(int controlNumber, int nSelections)
	{
		Logger.Debug("Updating select link for control number " + controlNumber + " with " + nSelections + " selections");

		if (controlNumber == _sourceSelectionControlNumber)
		{
			updateSourceSelectionMaximum(nSelections);
		}
		else if (controlNumber == _toneSelectionControlNumber)
		{
			updateToneSelectionMaximum(nSelections);
		}
	}

	/// <summary>
	/// Update a selection name for a selector control link according to control number, index and value. 
	/// </summary>
	/// <param name="controlNumber">The control control number to update</param>
	/// <param name="selectionNumber">The index of the name to update</param>
	/// <param name="selectionName">The new name for the selection</param>
	private void setSelectionName(int controlNumber, int selectionNumber, string selectionName)
	{
		Logger.Debug("Updating selection for control number " + controlNumber + " selection " + selectionNumber + " to name " + selectionName);

		if (controlNumber == _sourceSelectionControlNumber)
		{
			updateSourceSelectionName(selectionNumber, selectionName);
		}
		else if (controlNumber == _toneSelectionControlNumber)
		{
			updateToneSelectionName(selectionNumber, selectionName);
		}
	}

	private string message = "";
	private bool messageInProgress = false;

	/// <summary>
	/// This method processes one byte from the Ethernet interface when the PC receives it from HAL.
	/// </summary>
	/// <param name="data">The data byte to process</param>
	protected override void OnByteIn(byte data)
	{
		Logger.Debug("OnByteIn: "+data);
		base.OnByteIn(data);
		if (data == '<')
		{
			// Start a new message
			message = "";
			messageInProgress = true;
		}
		else if (data == '>')
		{
			// Finish the current message
			messageInProgress = false;
			processMessage(message);
		}
		else if (messageInProgress)
		{
			// Message is in progress - add the new character to it
			message += (char)data;
		}
	}
}
